@@ -47,7 +47,7 @@ from threading import Thread
 
 import requests
 import urllib2
-from poster.encode import multipart_encode
+from poster.encode import multipart_encode, MultipartParam
 from poster.streaminghttp import register_openers
 
 
@@ -56,8 +56,8 @@ try:
 except ImportError:
     import json
 
-from bigml.util import invert_dictionary, localize, is_url, \
-    clear_console_line, reset_console_line, console_log
+from bigml.util import (invert_dictionary, localize, is_url,
+                        clear_console_line, reset_console_line, console_log)
 from bigml.util import DEFAULT_LOCALE
 
 register_openers()
@@ -180,6 +180,21 @@ def get_evaluation_id(evaluation):
     return get_resource(EVALUATION_RE, evaluation)
 
 
+def get_status(resource):
+    """Extracts status info if present or sets the default if public
+
+    """
+    if not isinstance(resource, dict):
+        raise ValueError("We need a complete resource to extract its status")
+    if 'object' in resource:
+        resource = resource['object']
+    if not resource.get('private', True):
+        status = {'code': FINISHED}
+    else:
+        status = resource['status']
+    return status
+
+
 ##############################################################################
 #
 # Patch for requests
@@ -623,28 +638,47 @@ class BigML(object):
             LOGGER.error("Wrong resource id")
             return
         resource = self._get("%s%s" % (self.url, resource_id))
-        code = resource['object']['status']['code']
+        status = get_status(resource)
+        code = status['code']
         if code in STATUSES:
             return STATUSES[code]
         else:
             return "UNKNOWN"
 
-    def check_resource(self, resource, get_method):
-        """Waits until a resource is finshed.
+    def check_resource(self, resource, get_method,
+                       query_string='', wait_time=1):
+        """Waits until a resource is finished.
+
+           Given a resource and its corresponding get_method
+               source, api.get_source
+               dataset, api.get_dataset
+               model, api.get_model
+               prediction, api.get_prediction
+               evaluation, api.get_evaluation
+           it calls the get_method on the resource with the given query_string
+           and waits with sleeping intervals of wait_time
+           until the resource is in a final state (either FINISHED
+           or FAULTY)
 
         """
-
+        kwargs = {}
         if isinstance(resource, basestring):
-            resource = get_method(resource)
+            if not EVALUATION_RE.match(resource):
+                kwargs = {'query_string': query_string}
+            resource = get_method(resource, **kwargs)
+        else:
+            if not EVALUATION_RE.match(self.get_resource_id(resource)):
+                kwargs = {'query_string': query_string}
 
         while True:
-            status = resource['object']['status']
+            status = get_status(resource)
             code = status['code']
             if code == FINISHED:
                 return resource
             elif code == FAULTY:
                 raise ValueError(status)
-            resource = get_method(resource)
+            time.sleep(wait_time)
+            resource = get_method(resource, **kwargs)
 
     def get_resource_id(self, resource):
         """Returns the resource id if it falls in one of the registered types
@@ -741,7 +775,7 @@ class BigML(object):
             'object': resource,
             'error': error}
 
-    def _upload_source(self, url, args, source):
+    def _upload_source(self, url, args, source, out=sys.stdout):
         """Uploads a source asynchronously.
 
         """
@@ -754,47 +788,15 @@ class BigML(object):
             if progress < 1.0:
                 source['object']['status']['progress'] = progress
 
-        code = HTTP_INTERNAL_SERVER_ERROR
-        error = {
-            "status": {
-                "code": code,
-                "message": "The resource couldn't be created"}}
-
-        body, headers = multipart_encode(args, cb=update_progress)
-        request = urllib2.Request(url, body, headers)
-
-        try:
-            response = urllib2.urlopen(request)
-            code = response.getcode()
-            if code == HTTP_CREATED:
-                location = response.headers['location']
-                content = response.read()
-                resource = json.loads(content, 'utf-8')
-                resource_id = resource['resource']
-                error = None
-        except ValueError:
-            LOGGER.error("Malformed response")
-        except urllib2.HTTPError, exception:
-            LOGGER.error("Error %s", exception.code)
-            code = exception.code
-            if code in [HTTP_BAD_REQUEST,
-                        HTTP_UNAUTHORIZED,
-                        HTTP_PAYMENT_REQUIRED,
-                        HTTP_NOT_FOUND]:
-                content = exception.read()
-                error = json.loads(content, 'utf-8')
-            else:
-                LOGGER.error("Unexpected error (%s)" % code)
-                code = HTTP_INTERNAL_SERVER_ERROR
-
-        except urllib2.URLError, exception:
-            LOGGER.error("Error establishing connection")
-            error = exception.args
-        source['code'] = code
-        source['resource'] = resource_id
-        source['location'] = location
-        source['object'] = resource
-        source['error'] = error
+        resource = self._process_source(source['resource'], source['location'],
+                                        source['object'],
+                                        args=args, progress_bar=True,
+                                        callback=update_progress, out=out)
+        source['code'] = resource['code']
+        source['resource'] = resource['resource']
+        source['location'] = resource['location']
+        source['object'] = resource['object']
+        source['error'] = resource['error']
 
     def _stream_source(self, file_name, args=None, async=False,
                        progress_bar=False, out=sys.stdout):
@@ -815,14 +817,25 @@ class BigML(object):
         elif 'source_parser' in args:
             args['source_parser'] = json.dumps(args['source_parser'])
 
-        code = HTTP_INTERNAL_SERVER_ERROR
         resource_id = None
         location = None
         resource = None
         error = None
 
+        try:
+            if isinstance(file_name, basestring):
+                args.update({os.path.basename(file_name):
+                             open(file_name, "rb")})
+            else:
+                args = args.items()
+                name = '<none>'
+                args.append(MultipartParam(name, filename=name,
+                                           fileobj=file_name))
+
+        except IOError, exception:
+            sys.exit("Error: cannot read training set. %s" % str(exception))
+
         if async:
-            args.update({os.path.basename(file_name): open(file_name, "rb")})
             source = {
                 'code': HTTP_ACCEPTED,
                 'resource': resource_id,
@@ -834,26 +847,32 @@ class BigML(object):
             upload_args = (self.source_url + self.auth, args, source)
             thread = Thread(target=self._upload_source,
                             args=upload_args,
-                            kwargs={})
+                            kwargs={'out': out})
             thread.start()
             return source
+        return self._process_source(resource_id, location, resource,
+                                    args=args, progress_bar=progress_bar,
+                                    callback=draw_progress_bar, out=out)
 
+    def _process_source(self, resource_id, location, resource,
+                        args=None, progress_bar=False, callback=None,
+                        out=sys.stdout):
+        """Creates a new source.
+
+        """
+        code = HTTP_INTERNAL_SERVER_ERROR
         error = {
             "status": {
                 "code": code,
                 "message": "The resource couldn't be created"}}
 
-        try:
-            args.update({os.path.basename(file_name): open(file_name, "rb")})
-        except IOError:
-            sys.exit("Error: cannot read training set")
-
-        if progress_bar:
-            body, headers = multipart_encode(args, cb=draw_progress_bar)
+        if progress_bar and callback is not None:
+            body, headers = multipart_encode(args, cb=callback)
         else:
             body, headers = multipart_encode(args)
 
         request = urllib2.Request(self.source_url + self.auth, body, headers)
+
         try:
             response = urllib2.urlopen(request)
             clear_console_line(out=out)
@@ -913,7 +932,7 @@ class BigML(object):
            The source parameter should be a string containing the
            source id or the dict returned by create_source.
            As source is an evolving object that is processed
-           until it reaches the FINISHED or FAULTY state, the function will
+           until it reaches the FINISHED or FAULTY state, thet function will
            return a dict that encloses the source values and state info
            available at the time it is called.
 
@@ -929,7 +948,7 @@ class BigML(object):
         """
         source = self.get_source(source)
         return (source['code'] == HTTP_OK and
-                source['object']['status']['code'] == FINISHED)
+                get_status(source)['code'] == FINISHED)
 
     def list_sources(self, query_string=''):
         """Lists all your remote sources.
@@ -1006,7 +1025,7 @@ class BigML(object):
         """
         resource = self.get_dataset(dataset)
         return (resource['code'] == HTTP_OK and
-                resource['object']['status']['code'] == FINISHED)
+                get_status(resource)['code'] == FINISHED)
 
     def list_datasets(self, query_string=''):
         """Lists all your datasets.
@@ -1079,7 +1098,7 @@ class BigML(object):
         """
         resource = self.get_model(model)
         return (resource['code'] == HTTP_OK and
-                resource['object']['status']['code'] == FINISHED)
+                get_status(resource)['code'] == FINISHED)
 
     def list_models(self, query_string=''):
         """Lists all your models.
